#include <stdio.h>
#include <vector>
#include "sptr.h"

using namespace std;

class Texture : public Base {
public:
	Texture() {
		static int counter = 0;
		id = counter++;
		printf( "Texture N%d created\n",id );
	}
	virtual ~Texture() {
		printf( "Texture N%d destroyed\n",id );
	}
private:
	int id;
};

class Material : public Base {
public:
	Material() {
		// Allocate texture.
		m_texture = new Texture;
	}

	// Set/Get texture pointer.
	void texture( Texture *tex ) { m_texture = tex;	} // Release old texture, AddRef new texture.
	Texture* texture() const { return m_texture; }
private:
	smart_ptr<Texture> m_texture;
};

void	main()	{
	typedef vector<smart_ptr<Material> > Materials;
	Materials materials;

	// Create some materials and put them in array.
	materials.push_back( new Material );
	materials.push_back( new Material );

	Material *mat1 = new Material;
	Material *mat2 = new Material;
	
	Texture *tex = new Texture;
	// Set new texture for mat1, original texture of mat1 destroyed.
	mat1->texture( tex );

	// Share texture from mat1, original texture of mat2 destroyed, tex have refCount = 2.
	mat2->texture( mat1->texture() );

	materials.push_back( mat1 );
	materials.push_back( mat2 );

	// Deallocate all materials and textures.
	materials.clear();
}